@@ -19,19 +19,17 @@ import tempfile
 import time
 from urllib import parse as urlparse
 
+from ironic_lib import disk_utils
 from ironic_lib import exception
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log
-from oslo_utils import units
 import requests
 
-from ironic_python_agent import disk_utils
 from ironic_python_agent import errors
 from ironic_python_agent.extensions import base
 from ironic_python_agent import hardware
 from ironic_python_agent import partition_utils
-from ironic_python_agent import qemu_img
 from ironic_python_agent import utils
 
 CONF = cfg.CONF
@@ -49,6 +47,93 @@ def _image_location(image_info):
     return os.path.join(tempfile.gettempdir(), image_info['id'])
 
 
+def _verify_basic_auth_creds(user, password, image_id):
+    """Verify the basic auth credentials used for image download are present.
+
+    :param user: Basic auth username
+    :param password: Basic auth password
+    :param image_id: id of the image that is being acted upon
+
+    :raises ImageDownloadError if the credentials are not present
+    """
+    expected_creds = {'user': user, 'password': password}
+    missing_creds = []
+    for key, value in expected_creds.items():
+        if not value:
+            missing_creds.append(key)
+    if missing_creds:
+        raise errors.ImageDownloadError(
+            image_id,
+            "Missing {} fields from HTTP(S) "
+            "basic auth config".format(missing_creds)
+        )
+
+
+def _gen_auth_from_image_info_user_pass(image_info, image_id):
+    """This function is used to pass the credentials to the chosen
+
+       credential verifier and in case the verification is successful
+       generate the compatible authentication object that will be used
+       with the request(s). This function handles the authentication object
+       generation for authentication strategies that are username+password
+       based. Credentials are collected via image_info.
+
+    :param image_info: Image information dictionary.
+    :param image_id: id of the image that is being acted upon
+
+    :return: Authentication object used directly by the request library
+    :rtype: requests.auth.HTTPBasicAuth
+    """
+    image_server_user = None
+    image_server_password = None
+
+    if image_info.get('image_server_auth_strategy') == 'http_basic':
+        image_server_user = image_info.get('image_server_user')
+        image_server_password = image_info.get('image_server_password')
+        _verify_basic_auth_creds(
+            image_server_user,
+            image_server_password,
+            image_id
+        )
+    else:
+        return None
+
+    return requests.auth.HTTPBasicAuth(image_server_user,
+                                       image_server_password)
+
+
+def _gen_auth_from_oslo_conf_user_pass(image_id):
+    """This function is used to pass the credentials to the chosen
+
+       credential verifier and in case the verification is successful
+       generate the compatible authentication object that will be used
+       with the request(s). This function handles the authentication object
+       generation for authentication strategies that are username+password
+       based. Credentials are collected from the oslo.config framework.
+
+    :param image_id: id of the image that is being acted upon
+
+    :return: Authentication object used directly by the request library
+    :rtype: requests.auth.HTTPBasicAuth
+    """
+
+    image_server_user = None
+    image_server_password = None
+
+    if CONF.image_server_auth_strategy == 'http_basic':
+        _verify_basic_auth_creds(
+            CONF.image_server_user,
+            CONF.image_server_password,
+            image_id)
+        image_server_user = CONF.image_server_user
+        image_server_password = CONF.image_server_password
+    else:
+        return None
+
+    return requests.auth.HTTPBasicAuth(image_server_user,
+                                       image_server_password)
+
+
 def _download_with_proxy(image_info, url, image_id):
     """Opens a download stream for the given URL.
 
@@ -58,13 +143,31 @@ def _download_with_proxy(image_info, url, image_id):
 
     :raises: ImageDownloadError if the download stream was not started
              properly.
+
+    :return: HTTP(s) server response for the image/hash download request
+    :rtype: requests.Response
     """
+
     no_proxy = image_info.get('no_proxy')
     if no_proxy:
         os.environ['no_proxy'] = no_proxy
     proxies = image_info.get('proxies', {})
     verify, cert = utils.get_ssl_client_options(CONF)
     resp = None
+    image_download_attributes = {
+        "stream": True,
+        "proxies": proxies,
+        "verify": verify,
+        "cert": cert,
+        "timeout": CONF.image_download_connection_timeout
+    }
+    # NOTE(Adam) `image_info` is prioritized over `oslo.conf` for credential
+    # collection and auth strategy selection
+    auth_object = _gen_auth_from_image_info_user_pass(image_info, image_id)
+    if auth_object is None:
+        auth_object = _gen_auth_from_oslo_conf_user_pass(image_id)
+    if auth_object is not None:
+        image_download_attributes['auth'] = auth_object
     for attempt in range(CONF.image_download_connection_retries + 1):
         try:
             # NOTE(TheJulia) The get request below does the following:
@@ -80,9 +183,8 @@ def _download_with_proxy(image_info, url, image_id):
             # exactly just as the timeout value exists. The risk in transitory
             # failure is more so once we've started the download and we are
             # processing the incoming data.
-            resp = requests.get(url, stream=True, proxies=proxies,
-                                verify=verify, cert=cert,
-                                timeout=CONF.image_download_connection_timeout)
+            # B113 issue is covered is the image_download_attributs list
+            resp = requests.get(url, **image_download_attributes)  # nosec
             if resp.status_code != 200:
                 msg = ('Received status code {} from {}, expected 200. '
                        'Response body: {} Response headers: {}').format(
@@ -174,8 +276,7 @@ def _fetch_checksum(checksum, image_info):
         checksum, "Checksum file does not contain name %s" % expected_fname)
 
 
-def _write_partition_image(image, image_info, device, configdrive=None,
-                           source_format=None, is_raw=False, size=0):
+def _write_partition_image(image, image_info, device, configdrive=None):
     """Call disk_util to create partition and write the partition image.
 
     :param image: Local path to image file to be written to the partition.
@@ -186,10 +287,6 @@ def _write_partition_image(image, image_info, device, configdrive=None,
     :param configdrive: A string containing the location of the config
                         drive as a URL OR the contents (as gzip/base64)
                         of the configdrive. Optional, defaults to None.
-    :param source_format: The actual format of the partition image.
-                         Must be provided if deep image inspection is enabled.
-    :param is_raw: Ironic indicates the image is raw; do not convert it
-    :param size: Virtual size, in MB, of provided image.
 
     :raises: InvalidCommandParamsError if the partition is too small for the
              provided image.
@@ -209,9 +306,10 @@ def _write_partition_image(image, image_info, device, configdrive=None,
     cpu_arch = hardware.dispatch_to_managers('get_cpus').architecture
 
     if image is not None:
-        if size > int(root_mb):
+        image_mb = disk_utils.get_image_mb(image)
+        if image_mb > int(root_mb):
             msg = ('Root partition is too small for requested image. Image '
-                   'virtual size: {} MB, Root size: {} MB').format(size,
+                   'virtual size: {} MB, Root size: {} MB').format(image_mb,
                                                                    root_mb)
             raise errors.InvalidCommandParamsError(msg)
 
@@ -225,15 +323,12 @@ def _write_partition_image(image, image_info, device, configdrive=None,
                                             configdrive=configdrive,
                                             boot_mode=boot_mode,
                                             disk_label=disk_label,
-                                            cpu_arch=cpu_arch,
-                                            source_format=source_format,
-                                            is_raw=is_raw)
+                                            cpu_arch=cpu_arch)
     except processutils.ProcessExecutionError as e:
         raise errors.ImageWriteError(device, e.exit_code, e.stdout, e.stderr)
 
 
-def _write_whole_disk_image(image, image_info, device, source_format=None,
-                            is_raw=False):
+def _write_whole_disk_image(image, image_info, device):
     """Writes a whole disk image to the specified device.
 
     :param image: Local path to image file to be written to the disk.
@@ -241,40 +336,22 @@ def _write_whole_disk_image(image, image_info, device, source_format=None,
                        This parameter is currently unused by the function.
     :param device: The device name, as a string, on which to store the image.
                    Example: '/dev/sda'
-    :param source_format: The format of the whole disk image to be written.
-    :param is_raw: Ironic indicates the image is raw; do not convert it
+
     :raises: ImageWriteError if the command to write the image encounters an
              error.
-    :raises: InvalidImage if asked to write an image without a format when
-                          not permitted
     """
     # FIXME(dtantsur): pass the real node UUID for logging
     disk_utils.destroy_disk_metadata(device, '')
     disk_utils.udev_settle()
 
+    command = ['qemu-img', 'convert',
+               '-t', 'directsync', '-S', '0', '-O', 'host_device', '-W',
+               image, device]
+    LOG.info('Writing image with command: %s', ' '.join(command))
     try:
-        if is_raw:
-            # TODO(JayF): We should unify all these dd/convert_image calls
-            # into disk_utils.populate_image().
-            # NOTE(JayF): Since we do not safety check raw images, we must use
-            #  dd to write them to ensure maximum security. This may cause
-            #  failures in situations where images are configured as raw but
-            #  are actually in need of conversion. Those cases can no longer
-            #  be transparently handled safely.
-            LOG.info('Writing raw image %s to device %s', image, device)
-            disk_utils.dd(image, device)
-        else:
-            command = ['qemu-img', 'convert',
-                       '-t', 'directsync', '-S', '0', '-O', 'host_device',
-                       '-W']
-            if source_format:
-                command += ['-f', source_format]
-            command += [image, device]
-            LOG.info('Writing image with command: %s', ' '.join(command))
-            qemu_img.convert_image(image, device, out_format='host_device',
-                                   cache='directsync', out_of_order=True,
-                                   sparse_size='0',
-                                   source_format=source_format)
+        disk_utils.convert_image(image, device, out_format='host_device',
+                                 cache='directsync', out_of_order=True,
+                                 sparse_size='0')
     except processutils.ProcessExecutionError as e:
         raise errors.ImageWriteError(device, e.exit_code, e.stdout, e.stderr)
 
@@ -292,28 +369,14 @@ def _write_image(image_info, device, configdrive=None):
                         of the configdrive. Optional, defaults to None.
     :raises: ImageWriteError if the command to write the image encounters an
              error.
-    :raises: InvalidImage if the image does not pass security inspection
     """
     starttime = time.time()
     image = _image_location(image_info)
-    ironic_disk_format = image_info.get('disk_format')
-    is_raw = ironic_disk_format == 'raw'
-    # NOTE(JayF): The below method call performs a required security check
-    #             and must remain in place. See bug #2071740
-    source_format, size = disk_utils.get_and_validate_image_format(
-        image, ironic_disk_format)
-    size_mb = int((size + units.Mi - 1) / units.Mi)
-
     uuids = {}
     if image_info.get('image_type') == 'partition':
-        uuids = _write_partition_image(image, image_info, device,
-                                       configdrive,
-                                       source_format=source_format,
-                                       is_raw=is_raw, size=size_mb)
+        uuids = _write_partition_image(image, image_info, device, configdrive)
     else:
-        _write_whole_disk_image(image, image_info, device,
-                                source_format=source_format,
-                                is_raw=is_raw)
+        _write_whole_disk_image(image, image_info, device)
     totaltime = time.time() - starttime
     LOG.info('Image %(image)s written to device %(device)s in %(totaltime)s '
              'seconds', {'image': image, 'device': device,
@@ -352,7 +415,7 @@ def _get_algorithm_by_length(checksum):
 
     :param checksum: The requested checksum.
     :returns: A hashlib object based upon the checksum
-              or ValueError if the algorthm could not be
+              or ValueError if the algorithm could not be
               identified.
     """
     # NOTE(TheJulia): This is all based on SHA-2 lengths.
@@ -399,7 +462,7 @@ class ImageDownload(object):
     def __init__(self, image_info, time_obj=None):
         """Initialize an instance of the ImageDownload class.
 
-        Trys each URL in image_info successively until a URL returns a
+        Tries each URL in image_info successively until a URL returns a
         successful request code. Once the object is initialized, the user may
         retrieve chunks of the image through the standard python iterator
         interface until either the image is fully downloaded, or an error is
@@ -437,7 +500,7 @@ class ImageDownload(object):
                 image_info)
             retrieved_checksum = True
             if not algo:
-                # Override algorithm not suppied as os_hash_algo
+                # Override algorithm not supplied as os_hash_algo
                 self._hash_algo = _get_algorithm_by_length(
                     self._expected_hash_value)
         elif checksum:
@@ -447,7 +510,7 @@ class ImageDownload(object):
 
                 if not new_algo:
                     # Realistically, this should never happen, but for
-                    # compatability...
+                    # compatibility...
                     # TODO(TheJulia): Remove for a 2024 release.
                     self._hash_algo = hashlib.new('md5')  # nosec
                 else:
@@ -477,25 +540,6 @@ class ImageDownload(object):
                 self._expected_hash_value,
                 image_info)
 
-        # NOTE(dtantsur): verify that the user's input does not obviously
-        # contradict the actual value. It is especially easy to make such
-        # a mistake when providing a checksum URL.
-        if algo:
-            try:
-                detected_algo = _get_algorithm_by_length(
-                    self._expected_hash_value)
-            except ValueError:
-                pass  # an exotic algorithm?
-            else:
-                if detected_algo.name != algo:
-                    LOG.warning("Provided checksum algorithm %(provided)s "
-                                "does not match the detected algorithm "
-                                "%(detected)s. It may be a sign of a user "
-                                "error when providing the algorithm or the "
-                                "checksum URL.",
-                                {'provided': algo,
-                                 'detected': detected_algo.name})
-
         details = []
         for url in image_info['urls']:
             try:
@@ -632,7 +676,7 @@ def _download_image(image_info):
     totaltime = time.time() - starttime
     LOG.info("Image downloaded from %(image_location)s "
              "in %(totaltime)s seconds. Transferred %(size)s bytes. "
-             "Server originaly reported: %(reported)s.",
+             "Server originally reported: %(reported)s.",
              {'image_location': image_location,
               'totaltime': totaltime,
               'size': image_download.bytes_transferred,
@@ -662,7 +706,7 @@ def _validate_image_info(ext, image_info=None, **kwargs):
             msg = 'Image is missing \'{}\' field.'.format(field)
             raise errors.InvalidCommandParamsError(msg)
 
-    if type(image_info['urls']) != list or not image_info['urls']:
+    if not isinstance(image_info['urls'], list) or not image_info['urls']:
         raise errors.InvalidCommandParamsError(
             'Image \'urls\' must be a list with at least one element.')
 
@@ -709,7 +753,7 @@ def _validate_partitioning(device):
     try:
         # Ensure we re-read the partition table before we try to list
         # partitions
-        utils.execute('partprobe', device, run_as_root=True,
+        utils.execute('partprobe', device,
                       attempts=CONF.disk_utils.partprobe_attempts)
     except (processutils.UnknownArgumentError,
             processutils.ProcessExecutionError, OSError) as e:
@@ -810,7 +854,7 @@ class StandbyExtension(base.BaseAgentExtension):
 
         totaltime = time.time() - starttime
         LOG.info("Image streamed onto device %(device)s in %(totaltime)s "
-                 "seconds for %(size)s bytes. Server originaly reported "
+                 "seconds for %(size)s bytes. Server originally reported "
                  "%(reported)s.",
                  {'device': device, 'totaltime': totaltime,
                   'size': image_download.bytes_transferred,
@@ -843,45 +887,6 @@ class StandbyExtension(base.BaseAgentExtension):
         else:
             self.partition_uuids['root uuid'] = root_uuid
 
-    @base.async_command('cache_image', _validate_image_info)
-    def cache_image(self, image_info, force=False, configdrive=None):
-        """Asynchronously caches specified image to the local OS device.
-
-        :param image_info: Image information dictionary.
-        :param force: Optional. If True forces cache_image to download and
-                      cache image, even if the same image already exists on
-                      the local OS install device. Defaults to False.
-        :param configdrive: A string containing the location of the config
-                            drive as a URL OR the contents (as gzip/base64)
-                            of the configdrive. Optional, defaults to None.
-
-        :raises: ImageDownloadError if the image download fails for any reason.
-        :raises: ImageChecksumError if the downloaded image's checksum does not
-                  match the one reported in image_info.
-        :raises: ImageWriteError if writing the image fails.
-        """
-        LOG.debug('Caching image %s', image_info['id'])
-        device = hardware.dispatch_to_managers('get_os_install_device',
-                                               permit_refresh=True)
-
-        msg = 'image ({}) already present on device {} '
-
-        if self.cached_image_id != image_info['id'] or force:
-            LOG.debug('Already had %s cached, overwriting',
-                      self.cached_image_id)
-            # NOTE(dtantsur): backward compatibility
-            if configdrive is None:
-                configdrive = image_info.pop('configdrive', None)
-            self._cache_and_write_image(image_info, device, configdrive)
-            msg = 'image ({}) cached to device {} '
-
-        self._fix_up_partition_uuids(image_info, device)
-        result_msg = _message_format(msg, image_info, device,
-                                     self.partition_uuids)
-
-        LOG.info(result_msg)
-        return result_msg
-
     @base.async_command('prepare_image', _validate_image_info)
     def prepare_image(self, image_info, configdrive=None):
         """Asynchronously prepares specified image on local OS install device.
@@ -911,20 +916,16 @@ class StandbyExtension(base.BaseAgentExtension):
         device = hardware.dispatch_to_managers('get_os_install_device',
                                                permit_refresh=True)
 
-        requested_disk_format = image_info.get('disk_format')
-
+        disk_format = image_info.get('disk_format')
         stream_raw_images = image_info.get('stream_raw_images', False)
-
         # don't write image again if already cached
         if self.cached_image_id != image_info['id']:
             if self.cached_image_id is not None:
                 LOG.debug('Already had %s cached, overwriting',
                           self.cached_image_id)
 
-            if stream_raw_images and requested_disk_format == 'raw':
+            if stream_raw_images and disk_format == 'raw':
                 if image_info.get('image_type') == 'partition':
-                    # NOTE(JayF): This only creates partitions due to image
-                    #             being None
                     self.partition_uuids = _write_partition_image(None,
                                                                   image_info,
                                                                   device,
@@ -934,9 +935,6 @@ class StandbyExtension(base.BaseAgentExtension):
                     self.partition_uuids = {}
                     stream_to = device
 
-                # NOTE(JayF): Images that claim to be raw are not inspected at
-                #             all, as they never interact with qemu-img and are
-                #             streamed directly to disk unmodified.
                 self._stream_raw_image_onto_device(image_info, stream_to)
             else:
                 self._cache_and_write_image(image_info, device, configdrive)
