@@ -165,7 +165,7 @@ BLK_INCOMPLETE_DEVICE_TEMPLATE_SMALL = """
 # NOTE(dszumski): We include some partitions here to verify that
 # they are filtered out when not requested. It is assumed that
 # ROTA has been set to 0 on some software RAID devices for testing
-# purposes. In practice is appears to inherit from the underyling
+# purposes. In practice is appears to inherit from the underlying
 # devices, so in this example it would normally be 1.
 RAID_BLK_DEVICE_TEMPLATE = ("""
 {
@@ -490,6 +490,34 @@ LSHW_JSON_OUTPUT_V2 = ("""
       "serial" : "1234",
       "slot" : "NULL",
       "children" : [
+        {
+          "id": "firmware",
+          "class": "memory",
+          "claimed": true,
+          "description": "BIOS",
+          "vendor": "BIOSVNDR",
+          "physid": "0",
+          "version": "1.2.3",
+          "date": "03/30/2023",
+          "units": "bytes",
+          "size": 65536,
+          "capacity": 16777216,
+          "capabilities": {
+            "isa": "ISA bus",
+            "pci": "PCI bus",
+            "pnp": "Plug-and-Play",
+            "upgrade": "BIOS EEPROM can be upgraded",
+            "shadowing": "BIOS shadowing",
+            "cdboot": "Booting from CD-ROM/DVD",
+            "bootselect": "Selectable boot path",
+            "edd": "Enhanced Disk Drive extensions",
+            "acpi": "ACPI",
+            "usb": "USB legacy emulation",
+            "biosbootspecification": "BIOS boot specification",
+            "netboot": "Function-key initiated network service boot",
+            "uefi": "UEFI specification is supported"
+          }
+        },
         {
           "id" : "memory:0",
           "class" : "memory",
@@ -633,6 +661,60 @@ LSHW_JSON_OUTPUT_V2 = ("""
           "id" : "memory:5",
           "class" : "memory",
           "physid" : "2"
+        },
+        {
+          "id" : "network:0",
+          "class" : "network",
+          "handle" : "PCI:0000:00:14.3",
+          "description" : "Wireless interface",
+          "product" : "ABCD",
+          "vendor" : "ABCD",
+          "physid" : "14.3",
+          "businfo" : "pci@0000:00:14.3",
+          "logicalname" : "wlp0s20f3",
+          "width" : 64,
+          "clock" : 33000000,
+          "capabilities" : {
+            "pm" : "Power Management",
+            "msi" : "Message Signalled Interrupts",
+            "pciexpress" : "PCI Express",
+            "msix" : "MSI-X",
+            "bus_master" : "bus mastering",
+            "cap_list" : "PCI capabilities listing",
+            "ethernet" : true,
+            "physical" : "Physical interface",
+            "wireless" : "Wireless-LAN"
+          }
+        },
+        {
+          "id" : "network:1",
+          "class" : "network",
+          "handle" : "PCI:0000:00:1f.6",
+          "description" : "Ethernet interface",
+          "product" : "DCBA",
+          "vendor" : "DCBA",
+          "physid" : "1f.6",
+          "businfo" : "pci@0000:00:1f.6",
+          "logicalname" : "eth0",
+          "units" : "bit/s",
+          "capacity" : 1000000000,
+          "width" : 32,
+          "clock" : 33000000,
+          "capabilities" : {
+            "pm" : "Power Management",
+            "msi" : "Message Signalled Interrupts",
+            "bus_master" : "bus mastering",
+            "cap_list" : "PCI capabilities listing",
+            "ethernet" : true,
+            "physical" : "Physical interface",
+            "tp" : "twisted pair",
+            "10bt" : "10Mbit/s",
+            "10bt-fd" : "10Mbit/s (full duplex)",
+            "100bt" : "100Mbit/s",
+            "100bt-fd" : "100Mbit/s (full duplex)",
+            "1000bt-fd" : "1Gbit/s (full duplex)",
+            "autonegotiation" : "Auto-negotiation"
+          }
         }
       ]
     }
@@ -1682,7 +1764,7 @@ MULTIPATH_VALID_PATH = '%s is a valid multipath device path'
 MULTIPATH_INVALID_PATH = '%s is not a valid multipath device path'
 
 MULTIPATH_LINKS_DM = (
-    'SUPER_FRIENDLY_NAME %s ATA,INTEL SSDSC2CT06\n'
+    'SUPER_FRIENDLY_ NAME (12345) %s ATA, INTEL SSDSC2CT06\n'
     'size=56G features=\'1 retain_attached_hw_handler\' hwhandler=\'0\' wp=rw\n'  # noqa
     ' `-+- policy=\'service-time 0\' prio=1 status=active\n'
     '  `- 0:0:0:0 device s  8:0  active ready running\n'
